#include <kernel/universe.h>
#include <kernel/parser.h>
#include <kernel/physical.h>
#include <kernel/user.h>

inherit "/kernel/lib/command";
inherit "/kernel/lib/sprintf";
inherit "/kernel/lib/arrays";

static mapping new_passwords;

void init(object person) {
  ::init(person);
  person->add_grammar("quit", "", "quit", ({ }) );
  person->add_grammar("passwd", "", "passwd", ({ }) );
  person->add_grammar("save", "", "save", ({ }) );
  person->add_grammar("people", "", "people", ({ }) );
  person->add_grammar("commands", "", "commands", ({ }) );
  person->add_grammar("alias", "%word %text", "set_alias", ({ 0, 1 }) );
  person->add_grammar("alias", "", "show_aliases", ({ }) );
  person->add_grammar("unalias", "%word", "unalias", ({ 0 }) );
  person->add_grammar("describe", "%text", "set_description", ({ 0 }) );
  person->add_grammar("gender", "%gender", "set_gender", ({ 0 }) );
  person->add_grammar("help", "%text", "help", ({ 0 }) );
  person->add_grammar("help", "", "help_no_subject", ({ }) );
  new_passwords=([ ]);
}

void quit(object actor, string verb) {
  actor->save_user();
  actor->send_message("See you later...\n");
  actor->kill_me();
}

void save(object actor, string verb) {
  actor->save_user();
}

void passwd(object actor, string verb) {
  if (actor->query_password()!="") {
    actor->send_message("Enter old password: ");
    input_to_no_echo("cmd_passwd_old");
  } else {
    actor->send_message("Enter new password: ");
    input_to_no_echo("cmd_passwd_new1");
  }
}

void cmd_passwd_old(string s) {
  this_user()->send_message("\n");
  if (crypt(s, this_user()->query_password())==this_user()->query_password()) {
    this_user()->send_message("Enter new password: ");
    input_to_no_echo("cmd_passwd_new1");
  } else {
    this_user()->send_message("Password incorrect.  Sorry.\n");
  }
}

void cmd_passwd_new1(string s) {
  this_user()->send_message("\n");
  if (s!="") {
    new_passwords[this_user()->query_name()]=crypt(s);
    this_user()->send_message("Enter new password again: ");
    input_to_no_echo("cmd_passwd_new2");
  } else {
    this_user()->set_password("");
    this_user()->send_message("Password cleared\n");
  }
}

void cmd_passwd_new2(string s) {
  this_user()->send_message("\n");
  if (crypt(s, new_passwords[this_user()->query_name()])==new_passwords[this_user()->query_name()]) {
    this_user()->set_password(new_passwords[this_user()->query_name()]);
    this_user()->send_message("Password changed.\n");
    this_user()->save_user();
  } else {
    this_user()->send_message("Password not changed.\n");
  }
  new_passwords[this_user()->query_name()]=0;
}

void people(object actor, string verb) {
  object *objs;
  object room;
  int i;
  mixed *info;
  string line;

  objs=users();
  for (i=0;i<sizeof(objs);i++) {
    info=objs[i]->query_user_info();
    room=objs[i]->query_environment();
    if (actor->query_property("wizard")) {
      line=objs[i]->query_name()+" from "+info[USER_INFO_HOST]+", currently in '";
    } else {
      line=objs[i]->query_name()+", currently in '";
    }
    if (room) {
      line+=room->query_art_name();
    } else {
      line+="the void";
    }
    line+="'.\n";
    if (info[USER_INFO_IDLE]>120) {
      line+=sprintf("  (%s has been idle for %d minutes)\n", objs[i]->query_name(),
        info[USER_INFO_IDLE]/60);
    }
    actor->send_message(line);
  }
}

void commands(object actor, string verb) {
  mixed *grammar;
  string pattern;
  int i, j;
  string *names;

  names=({ });
  actor->send_message("Commands:\n");
  grammar=actor->query_grammar();
  for (i=0; i<sizeof(grammar); i++) {
    if (!grammar[i][GRAMMAR_INVIS]) {
      pattern="";
/*      for (j=0; j<sizeof(grammar[i][GRAMMAR_PATTERN]); j++) {
        pattern+=" "+grammar[i][GRAMMAR_PATTERN][j];
      } */
      names+=({ grammar[i][GRAMMAR_VERB]+pattern });
    }
  }
  names=sort_array(names);
  names=uniq_array(names);
  actor->send_message(sprintf("%*#a\n", actor->query_cols(), names));
}

void set_alias(object actor, string verb, object *x, object *y, string name, string value) {
  actor->set_alias(name, value);
}

void unalias(object actor, string verb, object *x, string name) {
  actor->set_alias(name, "");
}

void show_aliases(object actor, string verb) {
  mapping aliases;
  string *names;
  string *values;
  int i;

  aliases=actor->query_aliases();
  names=map_indices(aliases);
  values=map_values(aliases);
  for (i=0;i<sizeof(names);i++) {
    actor->send_message(names[i]+" : "+values[i]+"\n");
  }
}

void set_description(object actor, string verb, object *x, string description) {
  actor->set_property(PROP_DESC, description);
}

int get_gender_from_string(string text) {
  string g;

  g=lower_case(text);
  switch (g) {
    case "it": case "n": case "neuter": return 0;
    case "m": case "male": return 1;
    case "f": case "female": return 2;
    default: return -1;
  }
}

mixed *get_objs_from_gender(object actor, string text, mixed *grammar_line) {
  if (get_gender_from_string(text)!=-1) {
    return ({ 1, ({ }) });
  } else {
    return ({ -1000, ({ }) });
  }
}

void set_gender(object actor, string verb, object *x, string gender) {
  actor->set_gender(get_gender_from_string(gender));
}

void help(object actor, string verb, object *x, string subject) {
  string text;

  text=actor->get_command_help(subject, actor);
  if (text) {
    actor->more_string(text);
  }
}

void help_no_subject(object actor, string verb) {
  actor->send_message("Type 'commands' for a list of commands.\n'soul' for a list of soul commands.\n'help <command>' for help on a command.\n");
}
