#include "kernel/physical.h"

inherit "/kernel/lib/command";
inherit "/kernel/lib/names";
inherit strings "/kernel/lib/sprintf";

void init(object person) {
  ::init(person);
  person->add_grammar("look", "", "look", ({ }) );
  person->add_grammar("look", "%object", "look_at", ({ 0 }));
  person->add_grammar("look", "at %object", "look_at", ({ 1 }));
  person->add_grammar("examine", "%object", "look_at", ({ 0 }));
  person->add_grammar("inv", "", "inventory", ({ }));
  person->add_grammar("get", "%multinotheld", "get", ({ 0 }));
  person->add_grammar("get", "%multiinside from %container", "get_from", ({ 0, 2 }));
  person->add_grammar("drop", "%multiheld", "drop", ({ 0 }));
  person->add_grammar("wear", "%multiheld", "wear", ({ 0 }));
  person->add_grammar("give", "%multiheld to %person", "give", ({ 0, 2 }));
  person->add_grammar("put", "%multiheld in %container", "put", ({ 0, 2 }));
}

void look_at(object actor, string verb, object *objects, string x) {
  int i;

  if (sizeof(objects)>1) {
    for (i=0;i<sizeof(objects);i++) {
      actor->send_message(sprintf("%*=s", actor->query_cols(), objects[i]->query_name()+": \n"+objects[i]->query_description())+"\n");
    }
  } else {
    actor->send_message(sprintf("%*=s", actor->query_cols(), objects[0]->query_description())+"\n");
  }
}

void look(object actor, string verb) {
  look_at(actor, verb, ({actor->query_environment()}), "");
}

void inventory(object actor, string verb) {
  string result;
  object *inv, *carrying, *wearing;
  int i;

  inv=actor->query_shallow_inventory();
  carrying=({ });
  wearing=({ });
  for (i=0; i<sizeof(inv); i++) {
    switch (inv[i]->query_property(PROP_RELATION)) {
      case REL_HELD: carrying+=({ inv[i] }); break;
      case REL_WORN: wearing+=({ inv[i] }); break;
    }
  }
  result="";
  if (sizeof(carrying)) {
    result=sprintf("%*=s", actor->query_cols(), "You are carrying "+name_list(carrying, nil, actor, 0, 1))+"\n";
  }
  if (sizeof(wearing)) {
    result+=sprintf("%*=s\n", actor->query_cols(), "You are wearing "+name_list(wearing , nil, actor, 0, 1));
  }
  actor->send_message(result);
}

void get(object actor, string verb, object *objects, string x) {
  int i;
  
  for (i=0;i<sizeof(objects);i++) {
    if (objects[i]!=actor) { /* Someone typing "get all" probably doesn't
                                      really mean 'all'... */
      objects[i]->move(actor, actor);
      objects[i]->set_property(PROP_RELATION, REL_HELD);
    }
  }
}

void get_from(object actor, string verb, object *objects, object *container, string x, string y) {
  int i;
  
  for (i=0;i<sizeof(objects);i++) {
    if (objects[i]!=actor) { /* Someone typing "get all from here" probably 
				doesn't really mean 'all'... */
      objects[i]->move(actor, actor);
      objects[i]->set_property(PROP_RELATION, REL_HELD);
    }
  }
}

void drop(object actor, string verb, object *objects, string x) {
  int i;

  for (i=0;i<sizeof(objects);i++) {
    objects[i]->move(actor->query_environment(), actor);
  }
}

void wear(object actor, string verb, object *objects, string x) {
  int i;

  for (i=0;i<sizeof(objects);i++) {
    objects[i]->set_property(PROP_RELATION, REL_WORN);
  }
}

void give(object actor, string verb, object *objects, object *person, string x, string y) {
  int i;

  for (i=0;i<sizeof(objects);i++) {
    if (objects[i]->is_inside(actor)) {
      objects[i]->move(person[0], actor);
      objects[i]->set_property(PROP_RELATION, REL_HELD);
    }
  }
}

void put(object actor, string verb, object *objects, object *container, string x, string y) {
  int i;

  for (i=0;i<sizeof(objects);i++) {
    if (objects[i]->is_inside(actor)) {
      objects[i]->move(container[0], actor);
      if (objects[i]) {
	/* When the move happens, event_move() is called.  This can do
	   anything, including destructing the object. */
        objects[i]->set_property(PROP_RELATION, REL_HELD);
      }
    }
  }
}
