#include <kernel/parser.h>
#include <kernel/universe.h>
#include <kernel/physical.h>
#define SOUL_DIR "/kernel/data/soul"

inherit "/kernel/lib/command";
inherit "/kernel/lib/arrays";
inherit "/kernel/lib/sprintf";
inherit "/kernel/lib/names";

mapping global_adverbs;
mapping global_messages;
mapping private_adverbs;
mapping private_messages;
mapping global_patterns;
mapping private_patterns;

void create() {
  ::create();
  global_adverbs=([ ]);
  global_messages=([ ]);
  private_adverbs=([ ]);
  private_messages=([ ]);
  global_patterns=([ ]);
  private_patterns=([ ]);
}

void add_soul_command(object person, string *soul_info, int is_private) {
  string verb;
  string *command_info;
  string pattern;
  int *params;
  int i, j;

  if (is_private) {
    if (!private_adverbs[person]) {
      private_adverbs[person]=([ ]);
    }
    if (!private_messages[person]) {
      private_messages[person]=([ ]);
    }
    if (!private_patterns[person]) {
      private_patterns[person]=([ ]);
    }
  }
  verb=soul_info[0];
  i=1;
  if (is_private) {
    private_patterns[person][verb]=({ });
  } else {
    global_patterns[verb]=({ });
  }
  while (i<sizeof(soul_info) && (soul_info[i]!="*")) {
    /* Load the command information */
    command_info=explode(":"+soul_info[i], ":");
    pattern=command_info[0];
    command_info=explode(command_info[1], ",");
    params=({ });
    for (j=0; j<sizeof(command_info); j++) {
      params+=({ (int)command_info[j] });
    }
    /* Soul commands don't show up in the command list */
    if (is_private) {
      person->add_grammar(verb, pattern, "private_soul_command", params, 1);
    } else {
      person->add_grammar(verb, pattern, "global_soul_command", params, 1);
    }
    if (is_private) {
      private_patterns[person][verb]+=({ pattern });
    } else {
      global_patterns[verb]+=({ pattern });
    }
    i++;
  }
  i++;
  while (i<sizeof(soul_info) && (soul_info[i]!="*")) {
    /* Load the adverbs */
    if (is_private) {
      private_adverbs[person][verb]=explode(soul_info[i], ",");
    } else {
      global_adverbs[verb]=explode(soul_info[i], ",");
    }
    i++;
  }
  i++;
  if (is_private) {
    private_messages[person][verb]=({ });
  } else {
    global_messages[verb]=({ });
  }
  while (i<sizeof(soul_info)) {
    /* Load the messages */
    if (is_private) {
      private_messages[person][verb]+=({ explode(":"+soul_info[i]+":", ":") });
    } else {
      global_messages[verb]+=({ explode(":"+soul_info[i]+":", ":") });
    }
    i++;
  }
}

void add_soul_dir(string path, object person, int is_private) {
  mixed *file_info;
  string *soul_info;
  int i;

  file_info=get_dir(path+"/*.s");
  if (sizeof(file_info[0])) {
    depends_on(path);
  }
  for (i=0;i<sizeof(file_info[0]);i++) {
    soul_info=explode(read_file(path+"/"+file_info[0][i]), "\n");
    add_soul_command(person, soul_info, is_private);
  }
}

void remove_soul_command(object person, string *soul_info, int is_private) {
  string verb;

  verb=soul_info[0];
  if (is_private) {
    private_adverbs[person][verb]=nil;
    private_patterns[person][verb]=nil;
    private_messages[person][verb]=nil;
  } else {
    global_adverbs[verb]=nil;
    global_patterns[verb]=nil;
    global_messages[verb]=nil;
  }
  person->remove_command(this_object(), verb);
}

void remove_soul_dir(string path, object person, int is_private) {
  mixed *file_info;
  string *soul_info;
  int i;

  file_info=get_dir(path+"/*.s");
  for (i=0;i<sizeof(file_info[0]);i++) {
    soul_info=explode(read_file(path+"/"+file_info[0][i]), "\n");
    remove_soul_command(person, soul_info, is_private);
  }
}

void init(object person) {
  ::init(person);
  add_soul_dir(SOUL_DIR, person, 0);
  if (person->query_home_dir()) {
    add_soul_dir(person->query_home_dir()+"/soul", person, 1);
  }
  person->add_grammar("soul", "", "soul", ({ }) );
  person->add_grammar("make_soul", "%word", "make_soul", ({ 0 }) );
}

void make_soul(object actor, string verb, object *x, string name) {
  object maker;

  maker=find_object("/kernel/obj/soul_maker");
  maker=clone_object(maker);
  maker->new_soul(actor, name);
}

string *list_soul_dir(string path) {
  mixed *file_info;
  string name;
  int i;
  string *names;

  names=({ });
  file_info=get_dir(path+"/*.s");
  for (i=0;i<sizeof(file_info[0]);i++) {
    sscanf(file_info[0][i], "%s.s", name);
    names+=({ name });
  }
  return names;
}

void soul(object actor, string verb) {
  mixed *grammar;
  string *names;
  int i;

  grammar=actor->query_grammar();
  names=({ });
  for (i=0;i<sizeof(grammar);i++) {
    if (grammar[i][GRAMMAR_OBJ]==this_object()) {
      names+=({ grammar[i][GRAMMAR_VERB] });
    }
  }
  names-=({ "soul", "make_soul" });
  names=uniq_array(names);
  names=sort_array(names);
  actor->send_message(sprintf("%*#a\n", actor->query_cols(), names));
}

mixed get_objs_from_adverb(object actor, string text, mixed *grammar_line) {
  mapping adverbs;

  if (grammar_line[GRAMMAR_FUNCTION]=="private_soul_command") {
    adverbs=private_adverbs[actor];
  } else {
    adverbs=global_adverbs;
  }
  if (member_array(text, adverbs[grammar_line[GRAMMAR_VERB]])!=-1) {
    return ({ 1, ({ }) });
  } else {
    return ({ -1000, ({ }) });
  }
}

string make_message(object destination, string message, object person, object *targets, string adverb, string text) {
  string *bits;
  string result;
  int i;

  bits=explode(message, " ");
  result="";
  for (i=0; i<sizeof(bits); i++) {
    if (bits[i]=="%you") {
      result+=name_list(targets, destination, person, 0, person==destination);
    } else if (bits[i]=="%posyou") {
      result+=name_list(targets, destination, person, 1, person==destination);
    } else if (bits[i]=="%subjective") {
      result+=person->query_pronoun(PRON_SUBJ);
    } else if (bits[i]=="%objective") {
      result+=person->query_pronoun(PRON_OBJ);
    } else if (bits[i]=="%self") {
      result+=person->query_pronoun(PRON_REFL);
    } else if (bits[i]=="%name") {
      result+=person->query_art_name();
    } else if (bits[i]=="%adverb") {
      result+=adverb;
    } else if (bits[i]=="%text") {
      result+=text;
    } else if (bits[i]=="%possessive") {
      result+=person->query_pronoun(PRON_POSS);
    } else if (bits[i]=="%people") {
      result+=name_list(targets, nil, person, 0, person==destination);
    } else if (bits[i]=="%pospeople") {
      result+=name_list(targets, nil, person, 1, person==destination);
    } else {
      result+=bits[i];
    }
    if (i!=sizeof(bits)-1) {
      result+=" ";
    }
  }
  return result+".";
}

void soul_command(object actor, string verb, object *people, string adverb, string text, mapping soul_messages) {
  int index;
  string *m;
  object *others;
  int i;
  object *people_minus_actor;

  send_event((actor->near_objects()-people)+people, "soul",
             actor, verb, adverb, people, text);
  index=0;
  if (adverb!="") {
    index+=1;
  }
  if (sizeof(people)) {
    index+=2;
  }
  if (text && text != "") {
    index+=4;
  }
  m=soul_messages[verb][index];
  actor->send_message(make_message(actor, m[0], actor, people, adverb, text)+"\n");
  people_minus_actor=people-({ actor });
  if (sizeof(people_minus_actor)) {
    for (i=0;i<sizeof(people_minus_actor);i++) {
      people_minus_actor[i]->send_message(make_message(people_minus_actor[i], m[1], actor, people, adverb, text)+"\n");
    }
  }
  others=actor->query_environment()->query_shallow_inventory()-({actor})-people;
  for (i=0;i<sizeof(others);i++) {
    others[i]->send_message(make_message(others[i], m[2], actor, people, adverb, text)+"\n");
  }
}

void global_soul_command(object actor, string verb, object *x, object *people, object *y, string adverb, string z, string text) {
  soul_command(actor, verb, people, adverb, text, global_messages);
}

void private_soul_command(object actor, string verb, object *x, object *people, object *y, string adverb, string z, string text) {
  soul_command(actor, verb, people, adverb, text, private_messages[actor]);
}


string help_str(string verb, object actor) {
  string result;
  int i, j;
  string *pat;
  string adverb;
  string text;
  object *targets;
  object *in_room;
  object zombie1, zombie2;
  mapping adverbs;
  mapping patterns;
  mapping messages;
  object tmp;
  int index;

  if (verb=="soul" || verb=="make_soul") {
    return ::help_str(verb, actor);
  }
  if (private_patterns[actor][verb]) {
    adverbs=private_adverbs[actor];
    patterns=private_patterns[actor];
    messages=private_messages[actor];
  } else {
    adverbs=global_adverbs;
    patterns=global_patterns;
    messages=global_messages;
  }
  result=capitalize(verb)+" soul command\n\n";
  for (i=0;i<sizeof(patterns[verb]);i++) {
    pat=explode(patterns[verb][i], " ");
    result+=verb+make_syntax_string(pat)+"\n";
  }
  result+="\n";
  if (adverbs[verb]) {
    if (sizeof(adverbs[verb])) {
      result+=sprintf("Adverbs:\n  %*#a\n\n", actor->query_cols()-10, adverbs[verb]);
    }
  }
  if (adverbs[verb] && sizeof(adverbs[verb])) {
    adverb=adverbs[verb][0];
  } else {
    adverb="";
  }
  targets=({ });
  in_room=actor->query_environment()->query_shallow_inventory();
  if (in_room) {
    for (i=0;i<sizeof(in_room);i++) {
      if (in_room[i]!=actor && in_room[i]->query_property(UNIV_PROP_PERSON)) {
        targets+=({ in_room[i] });
      }
    }
  }
  if (sizeof(targets)>2) {
    for (i=0;i<2;i++) {
      index=random(sizeof(targets));
      tmp=targets[index];
      targets[index]=targets[i];
      targets[i]=tmp;
    }
  }
  zombie1=find_object("/kernel/obj/generic_object");
  zombie1=clone_object(zombie1);
  if (actor->query_name()=="Andrew") {
    zombie1->set_name("Ming");
    zombie1->set_gender(2);
  } else {
    zombie1->set_name("Andrew");
    zombie1->set_gender(1);
  }
  zombie1->set_article("");
  zombie1->move(nil, actor);
  zombie2=find_object("/kernel/obj/generic_object");
  zombie2=clone_object(zombie2);
  if (actor->query_name()=="Allana") {
    zombie2->set_name("Rabbit");
    zombie1->set_gender(1);
  } else {
    if (sizeof(targets) && targets[0]->query_name()=="Allana") {
      if (actor->query_name()=="Aria") {
        zombie2->set_name("Yakk");
        zombie1->set_gender(1);
      } else {
        zombie2->set_name("Aria");
        zombie1->set_gender(2);
      }
    } else {
      zombie2->set_name("Allana");
      zombie1->set_gender(2);
    }
  }
  zombie2->set_article("");
  zombie2->move(nil, actor);
  if (sizeof(targets)==0) {
    targets+=({ zombie1 });
    targets+=({ zombie2 });
  } else if (sizeof(targets)==1) {
    targets+=({ zombie1 });
  } else {
    targets=targets[0..1];
  }
  text="a hundred wombles";
  for (i=0;i<sizeof(messages[verb]);i++) {
    if (sizeof(messages[verb][i])>2) {
      if (messages[verb][i][0]!="") {
        result+="You: "+make_message(actor, messages[verb][i][0], actor, targets, adverb, text)+"\n";
      }
      if (messages[verb][i][1]!="") {
        result+="Targets: "+make_message(targets[0], messages[verb][i][1], actor, targets, adverb, text)+"\n";
      }
      if (messages[verb][i][2]!="") {
        result+="Others: "+make_message(nil, messages[verb][i][2], actor, targets, adverb, text)+"\n";
      }
      result+="\n";
    }
  }
  destruct_object(zombie1);
  destruct_object(zombie2);
  return result;
}

/*Format of the soul data files:
accuse		    ; The name of the command
:-1,-1,-1	    ; grammar lines.  The three numbers give the position of
:%person:-1,0,-1    ; adverb, person, and text parts.
%person of %text:-1,0,2
%person %adverb:1,0,-1
*                   ; end of the grammar lines
happily,aggressively   ;list of adverbs for this command
*
                                     ;0
                                     ;1 with adverb
You accuse %person                   ;2 with person
You accuse %person %adverb           ;3 with adverb, person
                                     ;4 with text
                                     ;5 with adverb, text
You accuse %person of %text          ;6 with person, text
You accuse %person of %text %adverb  ;7 with adverb, person, text
*/
/* The tokens in the messages are:
   %you:	Fred and you
   %posyou:	Fred and your
   %subjective:	he, she, it
   %objective:	him, her, it
   %self:	himself, herself, itself
   %name:	(the user's name)
   %adverb:	(the adverb)
   %text:	(the text)
   %possessive:	his, her, its
   %people:	Fred and John
   %pospeople:	Fred and John's
*/
