inherit "/kernel/lib/command";
inherit "/kernel/lib/names";

mapping converse_targets;

void init(object person) {
  if (!converse_targets) {
    converse_targets=([ ]);
  }
  ::init(person);
  person->add_grammar("say", "%text", "say", ({ 0 }) );
  person->add_grammar("say", "to %multiperson %text", "say_to", ({ 1, 2 }));
  person->add_grammar("say", "%text to %multiperson", "say_to", ({ 2, 0 }));
  person->add_grammar("mutter", "%text", "say", ({ 0 }) );
  person->add_grammar("mutter", "to %multiperson %text", "say_to", ({ 1, 2 }));
  person->add_grammar("mutter", "%text to %multiperson", "say_to", ({ 2, 0 }));
  person->add_grammar("quote", "%text", "say", ({ 0 }) );
  person->add_grammar("quote", "to %multiperson %text", "say_to", ({ 0 }) );
  person->add_grammar("quote", "%text to %multiperson", "say_to", ({ 0 }) );
  person->add_grammar("sing", "%text", "say", ({ 0 }) );
  person->add_grammar("sing", "to %multiperson %text", "say_to", ({ 0 }) );
  person->add_grammar("sing", "%text to %multiperson", "say_to", ({ 0 }) );
  person->add_grammar("whisper", "to %person %text", "whisper", ({ 1, 2 }));
  person->add_grammar("whisper", "%text to %person", "whisper", ({ 2, 0 }));
  person->add_grammar("shout", "%text", "shout", ({ 0 }));
  person->add_grammar("tell", "%multiglobalperson %text", "tell", ({ 0, 1 }));
  person->add_grammar("emote", "%text", "emote", ({ 0 }) );
  person->add_grammar("emotes", "%text", "emotes", ({ 0 }) );
  person->add_grammar("converse", "", "converse", ({ }) );
  person->add_grammar("converse", "%multiperson", "converse_to", ({ 0 }) );
}

void say_to(object actor, string verb, object *people, object *x, string y, string text) {
  int i;
  object *people_minus_actor;

  if (verb=="quote" || verb=="sing") {
    text="\""+text+"\"";
  }
  send_event(actor->near_objects()-({actor})-people, "say", actor->query_name(), verb+"s to "+name_list(people, nil, actor, 0, 0), text);
  people_minus_actor=people-({ actor });
  for (i=0; i<sizeof(people_minus_actor); i++) {
    send_event(people_minus_actor[i], "say", actor->query_name(), verb+"s to "+name_list(people, people[i], actor, 0, 0), text);
  }
  send_event(actor, "say", "You", verb+" to "+name_list(people, nil, actor, 0, 1), text);
}

void say(object actor, string verb, object *x, string text) {
  if (verb=="quote" || verb=="sing") {
    text="\""+text+"\"";
  }
  send_event(actor->near_objects()-({actor}), "say", actor->query_name(), verb+"s", text);
  send_event(actor, "say", "You", verb, text);
}

void whisper(object actor, string verb, object *people, object *x, string y, string text) {
  send_event(actor->near_objects()-({actor, people}), "say", actor->query_name(), "whispers something to "+name_list(people, nil, actor, 0, 0), "");
  send_event(people-({ actor }), "say", actor->query_name(), "whispers to you", text);
  send_event(actor, "say", "You", "whisper to "+name_list(people, nil, actor, 0, 1), text);
}

void shout(object actor, string verb, object *x, string text) {
  send_event(actor->all_objects()-({actor}), "say", actor->query_name(), "shouts", text);
  send_event(actor, "say", "You", "shout", text);
}

void tell(object actor, string verb, object *people, object *x, string y, string text) {
  int i;
  object *people_minus_actor;

  people_minus_actor=people-({ actor });
  for (i=0; i<sizeof(people_minus_actor); i++) {
    send_event(people_minus_actor[i], "say", actor->query_name(), "tells "+name_list(people, people[i], actor, 0, 0), text);
  }
  send_event(actor, "say", "You", "tell "+name_list(people, nil, actor, 0, 1), text);
}

void emote(object actor, string verb, object *x, string text) {
  send_event(actor->near_objects()-({actor}), "emote", actor->query_name()+" "+text);
  send_event(actor, "emote", "You emote: "+actor->query_name()+" "+text);
}

void emotes(object actor, string verb, object *x, string text) {
  send_event(actor->near_objects()-({actor}), "emote", actor->query_name()+"'s "+text);
  send_event(actor, "emote", "You emote: "+actor->query_name()+"'s "+text);
}

void converse_prompt() {
  this_user()->send_message("] ");
}

void do_converse(string str) {
  object *ct;
  string targets;
  int i;

  if (str[0..0]=="!") {
    if (strlen(str)>1) {
      this_user()->execute_command(str[1..]);
    }
    converse_prompt();
    input_to("do_converse");
  } else if (str!="**") {
    ct=converse_targets[this_user()];
    if (ct) {
      targets="";
      for (i=0;i<sizeof(ct);i++) {
	if (i!=0) {
	  targets+=" and ";
	}
	targets+=ct[i]->query_name();
      }
      this_user()->execute_command("tell "+targets+" "+str);
    } else {
      this_user()->execute_command("say "+str);
    }
    converse_prompt();
    input_to("do_converse");
  }
}

void converse(object actor, string verb) {
  converse_targets[actor]=nil;
  converse_prompt();
  input_to("do_converse");
}

void converse_to(object actor, string verb, object *people, string x) {
  converse_targets[actor]=people;
  converse_prompt();
  input_to("do_converse");
}
