#include </kernel/parser.h>

inherit "/kernel/lib/strings";

static object *_command_people;

void create() {
  ::create();
  _command_people=({ });
}

void init(object person) {
  _command_people+=({person});
}

void uninit(object person) {
  person->remove_grammar(this_object());
  _command_people-=({person});
}

void upgraded() {
  /* When a command object is upgraded, it might have new commands
     defined or old ones removed.  We'll have to tell all user objects
     about it. */
  int i;
  object *people_c;

  if (_command_people) {
    people_c=_command_people;
    for (i=0;i<sizeof(people_c);i++) {
      if (people_c[i]) {
        this_object()->uninit(people_c[i]);
        this_object()->init(people_c[i]);
      }
    }
  }
}

string make_syntax_string(string *pat) {
  string result;
  int i, j;

  result="";
  for (i=0;i<sizeof(pat);i++) {
    result+=" ";
    if (pat[i]=="%person" || pat[i]=="%globalperson") {
	result+="<person>";
    }  else if (pat[i]=="%adverb") {
	result+="<adverb>";
    }  else if (pat[i]=="%multiperson" || pat[i]=="%multiglobalperson") {
	result+="<people>";
    } else if (pat[i]=="%object" || pat[i]=="%container" || 
		 pat[i]=="%inside" || pat[i]=="%outside" ||
		 pat[i]=="%outside" || pat[i]=="%held" || pat[i]=="%notheld") {
	result+="<object>";
    } else if (pat[i]=="%multi" || pat[i]=="%multiinside" ||
		 pat[i]=="%multioutside" || pat[i]=="%multioutside" ||
		 pat[i]=="%multiheld" || pat[i]=="%multinotheld") {
	result+="<objects>";
    } else if (pat[i]=="%thisobject") {
	result+=this_object()->query_ids()[0];
    } else if (pat[i]=="%file") {
	result+="<filename>";
    } else if (pat[i]=="%multifile") {
	result+="<filenames>";
    } else if (pat[i]=="%word") {
	result+="<word>";
    } else if (pat[i]=="%text") {
	result+="<text>";
    } else {
	result+=pat[i];
    }
  }
  return result;
}

string help_keyword() {
  string *bits;

  bits=explode(object_name(this_object()), "/");
  return bits[sizeof(bits)-1];
}

string help_str(string verb, object actor) {
  /* The default help behaviour for objects that don't provide their own
     help_str() function.  First look in /doc/help.  If the .hlp file
     isn't found there, look in the object's extras directory (found in
     the same directory as the object, with the same name (except for
     .c on the end of the object's file name - for example, help for
     the dispenser (/obj/dispenser) "grab" command will be found in
     /obj/dispenser/grab.hlp) */
   string path;
   mixed *grammar;
   string result;
   string *pattern;
   int i;

   path="/doc/help/"+help_keyword()+"/"+verb+".hlp";
   if (sizeof(get_dir(path)[0])) {
     result=read_file(path);
   } else {
     path=explode(object_name(this_object()), "#")[0]+"/"+verb+".hlp";
     if (sizeof(get_dir(path)[0])) {
       result=read_file(path);
     } else {
       grammar=actor->query_grammar();
       result=capitalize(verb)+" command\n\n";
       for (i=0;i<sizeof(grammar);i++) {
         if (grammar[i][GRAMMAR_VERB]==verb) {
  	   pattern=grammar[i][GRAMMAR_PATTERN];
  	   result+=verb+make_syntax_string(pattern)+"\n";
         }
       }
       if (result=="") {
         result=nil;
       }
     }
   }
   return result;
}
