#include <kernel/universe.h>
#include <kernel/physical.h>
#include <kernel/room.h>

inherit physical "/kernel/lib/physical";
inherit names "/kernel/lib/names";

void create() {
  physical::create();
  set_property(UNIV_PROP_ROOM, 1);
  set_property(UNIV_PROP_CONTAINER, 1);
}

string query_description() {
  string result;
  mapping exits;
  string *directions;
  object *objs, *inv;
  int i;

  result=::query_description()+"\n";
  inv=query_shallow_inventory();
  objs=({ });
  for (i=0; i<sizeof(inv); i++) {
    if (!inv[i]->query_property(PROP_ITEM)) {
      objs+=({inv[i]});
    }
  }
  objs-=({ this_user() });
  result+="You can see: "+capitalize(name_list(objs, nil, nil, 0, 1))+".\n";
  exits=query_property(PROP_EXITS);
  if (exits) {
    directions=map_indices(exits);
    result+="  Visible exits: "+implode(directions, ", ")+".";
  } else {
    result+="  There's no way out!";
  }
  return result;
}

void add_exit(string direction, string destination) {
  mapping exits;

  exits=query_property(PROP_EXITS);
  if (!exits) {
    exits=([ ]);
  }
  exits[direction]=destination;
  set_property(PROP_EXITS, exits);
}

void init(object person) {
  int i;
  mapping exits;
  string *directions;

  physical::init(person);
  exits=query_property(PROP_EXITS);
  directions=map_indices(exits);
  for (i=0;i<sizeof(directions);i++) {
    /* Exit commands are invisible */
    person->add_grammar(directions[i], "", "follow_exit", ({ }), 1);
  }
  if (person->query_property(UNIV_PROP_PERSON)) {
    person->execute_command("look");
  }
}

void follow_exit(object actor, string direction) {
  mapping exits;
  object destination;

  exits=query_property(PROP_EXITS);
  destination=find_object(exits[direction]);
  if (destination) {
    actor->move(destination, actor);
  } else {
    actor->send_message("There seems to be something blocking your way...\n");
  }
}

void add_item(string name, string description, mixed ids...) {
  object obj;
  int i;

  obj=find_object("/kernel/obj/item");
  if (obj) {
    obj=clone_object(obj);
    if (obj) {
      obj->set_property(PROP_ITEM, 1);
      obj->set_name(name);
      obj->set_description(description);
      if (sizeof(ids)) {
        obj->set_ids(ids[0]);
      } else {
        obj->set_ids(name);
      }
      obj->move(this_object(), this_object());
    }
  }
}

void add_object(string filename) {
  object obj;

  obj=find_object(filename);
  if (obj) {
    obj=clone_object(obj);
    if (obj) {
      obj->move(this_object(), this_object());
    }
  }
}
