#include <config.h>
#include <kernel/kernel.h>
#include <type.h>
#include <kernel/binary.h>

inherit "/kernel/lib/strings";

static int echo;
static int state;
static int connection_mode;
static string input_to_function;
static object input_to_object;
static string login_name;
static object user;

#define STATE_NORMAL 0
#define STATE_INPUT_TO 1

void create() {
  ::create();
  input_to_function="";
  input_to_object=nil;
  user=nil;
  login_name="";
}

void _input_to(object obj, string function, int noecho) {
  if (noecho) {
    send_message(0);
  }
  state=STATE_INPUT_TO;
  input_to_object=obj;
  input_to_function=function;
}

int open() {
  more_string(read_file("/data/banner.txt"));
  send_message("Enter your name: ");
  _input_to(this_object(), "receive_message_login", 0);
  return 1;
}

int close(int flag) {
  object user;

  user=USER_MANAGER->query_user(this_object());
  if (user) {
    user->go_netdead();
  }
  if (!flag) {
    destruct_object(this_object());
  }
}

void kill_me() {
  destruct_object(this_object());
}

void receive_message_input_to(string message) {
  state=STATE_NORMAL;
  send_message(1);
  if (input_to_object!=this_object()) {
    USER_MANAGER->query_user(this_object())->call_input_to(input_to_object, input_to_function, message);
  } else {
    call_other(input_to_object, input_to_function, message);
  }
}

void password_ok() {
  if (USER_MANAGER->query_connection(user)) {
    USER_MANAGER->set_connection(this_object(), user);
    user->relogin();
  } else {
    USER_MANAGER->set_connection(this_object(), user);
    user->login();
  }
}

void check_password() {
  string password;

  password=user->query_password();
  if (password && password!="") {
    send_message("Enter password: ");
    _input_to(this_object(), "enter_password", 1);
  } else {
    password_ok();
  }
}

void receive_message_login(string message) {
  string *bits;

  bits=explode(message, " ");
  if (sizeof(bits)!=1) {
    send_message("Your name must be one word.\nEnter your name: ");
    _input_to(this_object(), "receive_message_login", 0);
  } else {
    login_name=message;
    user=USER_MANAGER->find_user(login_name);
    if (user) {
      /* They were already logged in */
      check_password();
    } else {
      user=USER_MANAGER->load_user(login_name);
      if (user) {
	/* An existing user, not already logged in */
        check_password();
      } else {
	/* A new user, or a typo */
        send_message("That name does not exist.  Did you type it correctly? ");
	_input_to(this_object(), "receive_message_query_name", 0);
      }
    }
  }
}

void enter_password(string message) {
  send_message("\n");
  if (crypt(message, user->query_password())==user->query_password()) {
    password_ok();
  } else {
    user=nil;
    send_message("Enter your name: ");
    _input_to(this_object(), "receive_message_login", 0);
  }
}

void receive_message_query_name(string message) {
  string response;

  response=lower_case(message);
  if (response=="yes" || response=="y") {
    send_message("OK.  Creating a new user.\n");
    state=STATE_NORMAL;
    user=USER_MANAGER->create_new_user(login_name);
    USER_MANAGER->set_connection(this_object(), user);
    user->setup_new_user(login_name);
  } else if (response=="no" || response=="n") {
    send_message("Enter your name: ");
    _input_to(this_object(), "receive_message_login", 0);
  } else {
    send_message("Please respond 'yes' or 'no'.\n");
    send_message("That name does not exist.  Did you type it correctly? ");
    _input_to(this_object(), "receive_message_query_name", 0);
  }
}

void receive_message_normal(string message) {
  USER_MANAGER->query_user(this_object())->receive_message(message);
}

void receive_message(string message) {
  if (connection_mode==1 && echo) {
    send_message(message+"\n");
  }
  switch(state) {
    case STATE_INPUT_TO:
      receive_message_input_to(message);
      break;
    case STATE_NORMAL:
      receive_message_normal(message);
      break;
  }
  if (state==STATE_NORMAL) {
    if (USER_MANAGER->query_user(this_object())) {
      USER_MANAGER->query_user(this_object())->prompt();
    }
  }
}

void set_connection(int mode) {
  connection_mode=mode;
  echo=1;
}

void send_binary_packet(string data) {
  if (connection_mode==1) {
    data=" "+data+" ";
    data[0]=3;
    data[strlen(data)-1]=4;
    ::send_message(data);
  }
}

int send_message(mixed arg) {
  if (connection_mode==1 && typeof(arg)==T_STRING) {
    arg+=" ";
    arg[strlen(arg)-1]=4;
  }
  if (connection_mode==1 && typeof(arg)==T_INT) {
    echo=arg;
    if (arg) {
      send_binary_packet(BIN_ECHO_ON);
    } else {
      send_binary_packet(BIN_ECHO_OFF);
    }
    return 0;
  } else {
    return ::send_message(arg);
  }
}

int query_echo() {
  return (connection_mode==1 && echo);
}

void prepare_to_die() {
  USER_MANAGER->set_connection(this_object(), nil);
  send_binary_packet(BIN_USER_DISCONNECT);
}
