inherit "/kernel/lib/sprintf";
inherit "/kernel/lib/arrays";
inherit "/kernel/lib/files";

string verb;
string data;
string file_name;
object owner;
string *adverbs;
int got_adverb;
int *got_case;
string *message_descriptions;
string *message_target_descriptions;
int which_message;
int which_target_message;

void say(string str) {
  owner->send_message(sprintf("%*=s", owner->query_cols(), str)+"\n");
}

void prompt() {
  owner->send_message("] ");
}

void start_editing();
void start_messages();
void ask_for_messages();

void new_soul(object actor, string name) {
  mixed dir_info;

  data="";
  verb=name;
  file_name=actor->query_home_dir()+"/soul/"+name+".s";
  owner=actor;
  got_adverb=0;
  got_case=({ 0,0,0,0 });
  dir_info=get_dir(file_name);
  message_descriptions=({ "alone (for example 'smile')",
			  "with an adverb (for example 'smile happily')",
			  "with a person (for example 'smile at Ming')",
			  "with both an adverb and a person (for example 'smile happily at Ming')" });
  message_target_descriptions=({ "the user who entered the command",
				 "the target(s) of the command",
				 "everyone else in the room" });
  if (sizeof(dir_info[0])) {
    say("You already have a '"+name+"' command.  Are you sure you want to edit it?");
    prompt();
    input_to("check_edit");
  } else {
    start_editing();
  }
}

void check_edit(string answer) {
  answer=lower_case(answer);
  if (answer=="y" || answer=="yes") {
    start_editing();
  } else if (answer=="n" || answer=="no") {
    say("OK.  Editing aborted.");
  } else {
    say("Huh?  Please answer 'yes' or 'no'.");
    prompt();
    input_to("check_edit");
  }
}

void start_editing() {
  data=verb+"\n";
  say("Start with the syntax patterns.  Use %adverb or %people in place "+
      "of adverbs and people's names.  For example, the syntax patterns for "+
      "'smile' are:");
  say("at %people");
  say("%people");
  say("%adverb");
  say("%adverb at %people");
  say("at %people %adverb");
  say("Enter a blank line for 'no pattern', and '**' to finish entering "+
      "the syntax patterns.");
  prompt();
  input_to("get_syntax");
}

void get_syntax(string str) {
  string *bits;
  int adverb_pos, target_pos;
  int this_case;

  if (str!="**") {
    str=replace_string(str, "%people", "%multiperson");
    bits=explode(str, " ");
    adverb_pos=member_array("%adverb", bits);
    target_pos=member_array("%multiperson", bits);
    this_case=0;
    if (adverb_pos!=-1) {
      got_adverb=1;
      this_case++;
    }
    if (target_pos!=-1) {
      this_case+=2;
    }
    got_case[this_case]=1;
    data+=str+":"+adverb_pos+","+target_pos+",-1\n";
    prompt();
    input_to("get_syntax");
  } else {
    data+="*\n";
    if (got_adverb) {
      say("Now enter the adverbs, one per line.  Enter '**' to finish.");
      adverbs=({ });
      prompt();
      input_to("get_adverb");
    } else {
      data+="*\n";
      start_messages();
    }
  }
}

void get_adverb(string str) {
  if (str!="**") {
    adverbs+=({ str });
    prompt();
    input_to("get_adverb");
  } else {
    data+=implode(adverbs, ",")+"\n";
    data+="*\n";
    start_messages();
  }
}

void start_messages() {
  say("Finally, enter the messages to be printed when the command is used.");
  say("In these messages use the following tokens where people's names, "+
      "adverbs, and other variable data should be inserted:");
  say("%you        : (targets, pronoun)             Fred and you");
  say("%posyou     : (targets, possesive pronoun)   Fred and your");
  say("%subjective : (actor, subjective pronoun)    he, she, it");
  say("%objective  : (actor, objective pronoun)     him, her, it");
  say("%self       : (actor, reflexive pronoun)     himself, herself, itself");
  say("%name       : (actor's name)");
  say("%adverb     : (the adverb)");
  say("%possessive : (actor, possessive pronoun)    his, her, its");
  say("%people     : (targets' names)               Fred and John");
  say("%pospeople  : (targets' names, possessive)   Fred and John's");
  which_message=0;
  which_target_message=0;
  ask_for_messages();
}

void ask_for_messages() {
  say("Enter the description to be printed to "+
      message_target_descriptions[which_target_message]+
      " when the command is given "+
      message_descriptions[which_message]);
  prompt();
  input_to("get_message");
}

void all_done() {
  say("OK!  We're done!");
  ensure_path(file_name);
  remove_file(file_name);
  write_file(file_name, data);
  "/kernel/command/soul"->uninit(owner);
  "/kernel/command/soul"->init(owner);
  destruct_object(this_object());
}

void get_message(string str) {
  data+=str;
  switch(which_target_message) {
    case 0:
      if (which_message>=2) {
	which_target_message=1;
	data+=":";
      } else {
	which_target_message=2;
	data+="::";
      }
      break;
    case 1:
      which_target_message=2;
      data+=":";
      break;
    case 2:
      which_target_message=0;
      data+="\n";
      which_message++;
      while (which_message<4 && !got_case[which_message]) {
        data+="\n";
        which_message++;
      }
      break;
  }
  if (which_message!=4) {
    ask_for_messages();
  } else {
    all_done();
  }
}
