#include <config.h>
#include <kernel/kernel.h>
#include <kernel/physical.h>
#include <type.h>
#include <kernel/binary.h>

inherit "/kernel/lib/strings";

static int echo;
static int state;
static string input_to_function;
static object input_to_object;
static string login_name;
static object user;

#define STATE_NORMAL 0
#define STATE_INPUT_TO 1

void create() {
  ::create();
  input_to_function="";
  input_to_object=nil;
  user=nil;
  login_name="";
}

void _input_to(object obj, string function, int noecho) {
  if (noecho) {
    send_message(0);
  }
  state=STATE_INPUT_TO;
  input_to_object=obj;
  input_to_function=function;
}

int open() {
  return 1;
}

int close(int flag) {
  object user;

  user=USER_MANAGER->query_user(this_object());
  if (user) {
    destruct_object(user);
  }
  if (!flag) {
    destruct_object(this_object());
  }
}

void kill_me() {
  destruct_object(this_object());
}

void receive_message_input_to(string message) {
  state=STATE_NORMAL;
  send_message(1);
  if (input_to_object!=this_object()) {
    USER_MANAGER->query_user(this_object())->call_input_to(input_to_object, input_to_function, message);
  } else {
    call_other(input_to_object, input_to_function, message);
  }
}

void receive_message_connect(string name) {
  login_name=name;
  user=USER_MANAGER->find_user(login_name);
  if (user) {
      USER_MANAGER->set_connection(this_object(), user);
      user->relogin();
  } else {
    user=USER_MANAGER->load_user(login_name);
    if (user) {
      USER_MANAGER->set_connection(this_object(), user);
      user->login();
    } else {
      state=STATE_NORMAL;
      user=USER_MANAGER->create_new_user(login_name);
      USER_MANAGER->set_connection(this_object(), user);
      user->setup_new_user(login_name);
    }
  }
}

void receive_message_normal(string message) {
  USER_MANAGER->query_user(this_object())->receive_message(message);
}

void make_object(string data) {
  string file, name, description, id_string, extra_data;
  string *ids;
  string *bits;
  string article;
  object obj;
  int size, left;

  if (sscanf(data, "%s|%s|%s|%s|%s", file, name, description, id_string, extra_data)==5) {
    ids=explode(id_string, ":");
    bits=explode(name, " ");
    article=lower_case(bits[0]);
    if (article=="a" || article=="an" || article=="the") {
      name=implode(bits[1..], " ");
    } else {
      article=nil;
    }
    switch (file) {
      case "/obj/generic_object":
	obj=find_object("/kernel/obj/generic_object");
	obj=clone_object(obj);
	obj->set_ids(ids);
	obj->set_name(name);
	if (article) {
	  obj->set_article(article);
	}
	obj->set_description(description);
        obj->move(USER_MANAGER->query_user(this_object()), this_object());
	obj->set_property(PROP_RELATION, REL_HELD);
	break;
      case "/inherit/drink":
	sscanf(extra_data, "%d:%d", size, left);
	obj=find_object("/kernel/obj/food_drink");
	obj=clone_object(obj);
	obj->set_ids(ids);
	obj->set_property("food_type", 2);
	obj->set_name(name);
	if (article) {
	  obj->set_article(article);
	}
	obj->set_description(explode(description, "\\n")[0]);
	obj->set_size(size);
	obj->set_left(left);
        obj->move(USER_MANAGER->query_user(this_object()), this_object());
	obj->set_property(PROP_RELATION, REL_HELD);
	break;
      case "/inherit/food":
	sscanf(extra_data, "%d:%d", size, left);
	obj=find_object("/kernel/obj/food_drink");
	obj=clone_object(obj);
	obj->set_ids(ids);
	obj->set_property("food_type", 1);
	obj->set_name(name);
	if (article) {
	  obj->set_article(article);
	}
	obj->set_description(explode(description, "\\n")[0]);
	obj->set_size(size);
	obj->set_left(left);
        obj->move(USER_MANAGER->query_user(this_object()), this_object());
	obj->set_property(PROP_RELATION, REL_HELD);
	break;
      case "/inherit/clothing":
	obj=find_object("/kernel/obj/clothing");
	obj=clone_object(obj);
	obj->set_ids(ids);
	obj->set_name(name);
	if (article) {
	  obj->set_article(article);
	}
	obj->set_description(explode(description, "\\n")[0]);
        obj->move(USER_MANAGER->query_user(this_object()), this_object());
	if ((int)extra_data) {
	  obj->set_property(PROP_RELATION, REL_WORN);
	} else {
	  obj->set_property(PROP_RELATION, REL_HELD);
	}
	break;
    }
  }
}

void receive_message(string message) {
  string *bits;
  int i;
  string command, data;

  bits=explode(message, "\n");
  for (i=0;i<sizeof(bits);i++) {
    if (sscanf(bits[i], "%s:%s", command, data)==2) {
      switch (command) {
        case "connect":
  	receive_message_connect(data);
  	break;
        case "message":
          switch(state) {
            case STATE_INPUT_TO:
              receive_message_input_to(data);
              break;
            case STATE_NORMAL:
              receive_message_normal(data);
              break;
          }
          if (state==STATE_NORMAL) {
            if (USER_MANAGER->query_user(this_object())) {
              USER_MANAGER->query_user(this_object())->prompt();
            }
          }
  	  break;
        case "obj":
  	  make_object(data);
	  break;
      }
    }
  }
}

int send_message(mixed arg) {
  if (typeof(arg)==T_STRING) {
    arg=replace_string(arg, "\n", "\\n");
    ::send_message("print:"+arg+"\n");
  } else {
    return ::send_message(arg);
  }
}

void prepare_to_die() {
  ::send_message("quit:quit\n");
  USER_MANAGER->set_connection(this_object(), nil);
}

void send_tunnel_info(string info) {
  ::send_message(info);
}

string make_extra(object obj, string file) {
  string result;

  switch(file) {
    case "/kernel/obj/food_drink":
      if (obj->query_property("food_type")==1) {
	result="food:";
      } else {
	result="drink:";
      }
      return result+(string)obj->query_property("food_size")+":"+(string)obj->query_left();
    case "/kernel/obj/clothing":
      if (obj->query_property(PROP_RELATION)==REL_WORN) {
	return "1";
      } else {
	return "0";
      }
  }
  return "";
}

void quitting() {
  /* They're leaving.  We need to send the object they're carrying. */
  object *inv;
  int i;
  string file, name, description, ids, extra;
  object user;

  user=USER_MANAGER->query_user(this_object());
  inv=user->query_shallow_inventory();
  for (i=0;i<sizeof(inv);i++) {
    file=explode(object_name(inv[i]), "#")[0];
    name=replace_string(inv[i]->query_art_name(), "\n", "\\n");
    if (file=="/kernel/obj/food_drink") {
      description=replace_string(inv[i]->query_raw_description(), "\n", "\\n");
    } else {
      description=replace_string(inv[i]->query_description(), "\n", "\\n");
    }
    ids=implode(inv[i]->query_ids(), ":");
    extra=make_extra(inv[i], file);
    ::send_message("obj:"+file+"|"+name+"|"+description+"|"+ids+"|"+extra+"\n");
  }
}
