#include <config.h>
#include <status.h>
#include <kernel/kernel.h>
#include <kernel/universe.h>
#include <kernel/network.h>
#include <trace.h>
#include <type.h>

mapping inheritence;
mapping upgrading;
mapping last_upgraded;
int reboot_time;
string *upgraded_clones;

void message(string str) {
  send_message(ctime(time())[4..18]+" ** "+str+"\n");
  write_file("/log/debug", ctime(time())[4..18]+" ** "+str+"\n");
  if (this_user()) {
    this_user()->send_message(str+"\n");
  }
}

string lower_case(string s) {
  int i;

  for (i=0;i<strlen(s);i++) {
    if (s[i]<='Z' && s[i]>='A') {
      s[i]=s[i]-'A'+'a';
    }
  }
  return s;
}

void learn_inheritence(string from, string path) {
  if (from!=path) {
    if (!inheritence) {
      inheritence=([ ]);
    }
    if (!inheritence[path]) {
      inheritence[path]=({ });
    }
    inheritence[path]=(inheritence[path]-({ from }))+({ from });
  }
}

object compile_object(string name) {
  object o;

  o=::compile_object(name);
  /* Make sure create() gets called */
  call_other(o, "???");
  learn_inheritence(name, "/kernel/lib/auto");
  return o;
}

object load(string path) {
  object obj;

  obj=find_object(path);
  if (!obj) {
    obj=compile_object(path);
  }
  call_other(obj, "???");
  return obj;
}

void initialize() {
  object obj;

  remove_file("/log/debug");
  message("DGD "+status()[ST_VERSION]+" starting.");
  reboot_time=time();
  inheritence=([ "/kernel/sys/driver":({ }) ]);
  load(AUTO);
  obj=load(TELNET_PORT_OBJ);
  obj->create();
  obj=load(BINARY_PORT_OBJ);
  obj->create();
  obj=load(TUNNEL_PORT_OBJ);
  obj->create();
  load(UNIVERSE);
  message("Initialization complete.\n");
}

void restored() {
  message("DGD "+status()[ST_VERSION]+" restarting.");
  message("Restarted.\n");
}

string path_read(string path) {
  return path;
}

string path_write(string path) {
  string obj;
  string user;

  obj=explode(object_name(previous_object()), "#")[0];
  user=lower_case(this_user()->query_name());
  if (strlen(path)>=strlen("/users/"+user+"/") && path[0..strlen("/users/"+user+"/")-1]=="/users/"+user+"/") {
    /* A user can write to their own directory */
    return path;
  }
  if (path=="users/"+user) {
    /* A user can create or remove their own directory */
    return path;
  }
  if (strlen(obj)>=strlen("/kernel/") && obj[0..strlen("/kernel/")-1]=="/kernel/") {
    if (obj=="/kernel/obj/editor") {
      /* Editors can't write anywhere else */
      return nil;
    }
    if (obj[0..strlen("/kernel/command/")]=="/kernel/command/") {
      /* Can't let the commands do it either */
      return nil;
    }
    /* other kernel objects are trusted */
    return path;
  }
  return nil;
}

object call_object(string path) {
  return load(path);
}

object inherit_program(string from, string path, int priv) {
  if (!sscanf(path, "%*s/lib/")) {
    message("Illegal inherit: "+from+" attempting to inherit "+path);
    return nil;
  } else {
    learn_inheritence(from, path);
    return load(path);
  }
}

static string *_upgrade(string path) {
  string *inherited_by;
  object obj;
  string *done;
  int i;

  done=({ });
  if (inheritence) {
    obj=find_object(path);
    inherited_by=inheritence[path];
    if (!upgrading[path]) {
      upgrading[path]=1;
      if (inherited_by && sizeof(inherited_by)!=0) {
        /* It's a lib - destruct it and upgrade everything that inherits it */
        last_upgraded[path]=time();
        done+=({path});
        for (i=0;i<sizeof(inherited_by);i++) {
          done+=_upgrade(inherited_by[i]);
        }
      } else {
        /* It's an object - if a blueprint exists, recompile it.  If there is
           no blueprint, there's no point - it'll get compiled next time it
           is used. */
        if (obj) {
          last_upgraded[path]=time();
          upgraded_clones+=({path});
          done+=({path});
        }
      }
    }
  }
  return done;
}

do_compile(string *paths) {
  string this_path;
  mixed *dir_info;
  string *to_dest;
  string *to_compile;
  int i;
  object obj;

  to_dest=({ });
  to_compile=({ });
  while (sizeof(paths)) {
    this_path=paths[0];
    dir_info=get_dir(this_path+".c");
    if (sizeof(dir_info[0])) {
      if (sscanf(this_path, "%*s/lib/%*s")) {
	to_dest+=({ this_path });
      } else {
	to_compile+=({ this_path });
      }
    }
    paths-=({ this_path });
  }
  for (i=0;i<sizeof(to_dest);i++) {
    obj=find_object(to_dest[i]);
    if (obj) {
      destruct_object(obj);
    }
  }
  for (i=0;i<sizeof(to_compile);i++) {
    compile_object(to_compile[i]);
  }
}

static string *auto_upgrade() {
  mixed *dir_info;
  int last_upgrade, modified_time;
  string *done;
  string *files;
  string path;
  int i;

  files=({ })+map_indices(inheritence);
  for (i=0;i<sizeof(map_values(inheritence));i++) {
    files+=map_values(inheritence)[i];
  }
  done=({ });
  while (sizeof(files)) {
    path=files[0];
    if (last_upgraded[path]) {
      last_upgrade=last_upgraded[path];
    } else {
      last_upgrade=reboot_time;
    }
    dir_info=get_dir(path);
    if (!sizeof(dir_info[2]) || dir_info[1][0]==-2) {
      dir_info=get_dir(path+".c");
    }
    if (sizeof(dir_info[2])) {
      modified_time=dir_info[2][0];
      if (modified_time>last_upgrade) {
        done+=_upgrade(path);
      }
    }
    files-=({ path });
  }
  return done;
}

string *upgrade(string path) {
  string *done;

  upgraded_clones=({ });
  upgrading=([ ]);
  if (!last_upgraded) {
    last_upgraded=([ ]);
  }
  if (path) {
    /* Just upgrade that file (and anything that depends on it) */
    done=_upgrade(path);
  } else {
    /* Auto upgrade everything that has changed */
    done=auto_upgrade();
  }
  do_compile(done);
  /* This has to be done as a call_out, unfortunately.  The 'compile' kfun
     doesn't replace the objects with the new version until after the current
     thread has finished.  But we want to call upgraded() on the new version. */
  call_out("call_upgraded", 0, upgraded_clones+({ }));
  upgraded_clones=({ });
  return done;
}

void call_upgraded(string *paths) {
  /* Tell each object it has been upgraded.  Also move them to the void and back
     so uninit() and init() get called */
  int i, j;
  object obj;
  object *clones;

  for (i=0;i<sizeof(paths);i++) {
    obj=load(paths[i]);
    if (sscanf(paths[i], "%*s/obj/%*s")==2) {
      clones=obj->query_clones();
    } else {
      clones=({ obj });
    }
    if (clones) {
      for (j=0;j<sizeof(clones);j++) {
        obj=clones[j];
        if (obj->is_clone() && obj->is_physical()) {
          UNIVERSE->call_uninit(obj);
        }
        obj->upgraded();
        if (obj->is_clone() && obj->is_physical()) {
	  UNIVERSE->call_init(obj);
        }
      }
    }
  }
}

mapping query_inheritence() {
  return inheritence;
}

string path_include(string from, string path) {
  return path;
}

void recompile(object obj) {
  destruct_object(obj);
}

void interrupt() {
  message("Interrupt.");
  shutdown();
}

void compile_error(string file, int line, string err) {
  message("Compile error: "+file+":"+line+" "+err+"");
/*  last_upgraded[file]=-1; */
}

void runtime_error(string str, int caught, int ticks) {
  mixed **trace;
  int i;

  message("Runtime error: "+str);
  trace=call_trace();
  for (i=0;i<sizeof(trace)-1;i++) {
    message("  "+trace[i][TRACE_OBJNAME]+" file "+trace[i][TRACE_PROGNAME]+":"+trace[i][TRACE_LINE]+" in function "+trace[i][TRACE_FUNCTION]); 
  }
}

int compile_rlimits(string objname) {
  return 1;
}

int runtime_rlimits(object obj, int stack, int ticks) {
  return 1;
}

void remove_program(string path, int timestamp, int index) {
}
