#include <kernel/kernel.h>
#include <kernel/universe.h>
inherit "/kernel/lib/arrays";

static mapping connections;
static object *all_users;

void create() {
  connections=([ ]);
  all_users=({ });
}

object query_connection(object user) {
  /* Return the connection object for this user (nil if this user doesn't
     have a connection object */
  if (user) {
    return connections[user];
  } else {
    return nil;
  }
}

object query_user(object connection) {
  /*  Return the user object for this connection */
  int index;

  index=member_array(connection, map_values(connections));
  if (index!=-1) {
    return map_indices(connections)[index];
  } else {
    return nil;
  }
}

object find_user(string name) {
  /* If the user is logged in, return the user object.  Otherwise return
     nil */
  object *all_users;
  int i;

  all_users=users();
  for (i=0;i<sizeof(all_users);i++) {
    if (lower_case(all_users[i]->query_name())==lower_case(name)) {
      return all_users[i];
    }
  }
  return nil;
}

object load_user(string name) {
  /* Create a new user object and load it from the save file */
  object user;
  string filename;

  filename="/save/users/"+name[0..0]+"/"+name;
  if (sizeof(get_dir(filename)[0])) {
    user=find_object(USER);
    user=clone_object(user);
    user->restore_me(filename);
    all_users+=({user});
    user->set_property(UNIV_PROP_CHILDREN, ({ }));
    return user;
  } else {
    return nil;
  }
}

object create_new_user(string name) {
  /* Create a new user object, and set all the defaults */
  object user;

  user=find_object(USER);
  user=clone_object(user);
  user->set_name(name);
  all_users+=({user});
  return user;
}

void set_connection(object connection, object user) {
  object old_connection;

  old_connection=query_connection(user);
  if (old_connection) {
    destruct_object(old_connection);
  }
  connections[user]=connection;
}

void remove_user(object user) {
  all_users-=({user});
}

object *query_users() {
  return all_users;
}
