#include "kernel/physical.h"

inherit "/kernel/lib/command";
inherit "/kernel/lib/sprintf";
inherit "/kernel/lib/arrays";
inherit "/kernel/lib/files";

void init(object person) {
  ::init(person);
  person->add_grammar("man", "%text", "man", ({ 0 }) );
  person->add_grammar("cd", "%file", "cd", ({ 0 }) );
  person->add_grammar("ls", "", "ls", ({ }) );
  person->add_grammar("ls", "%file", "ls_path", ({ 0 }) );
  person->add_grammar("edit", "%text", "edit", ({ 0 }) );
  person->add_grammar("ed", "%text", "edit", ({ 0 }) );
  person->add_grammar("more", "%file", "more_file", ({ 0 }) );
  person->add_grammar("more", "%object", "more_object", ({ 0 }) );
  person->add_grammar("mkdir", "%text", "mkdir", ({ 0 }) );
  person->add_grammar("rmdir", "%text", "rmdir", ({ 0 }) );
}

void cd(object actor, string verb, object *x, string path) {
  actor->set_current_directory(path);
}

void ls(object actor, string verb) {
  mixed **dir_info;
  int i;
  string *names;

  dir_info=get_dir(actor->real_dir("*"));
  names=({ });
  for (i=0; i<sizeof(dir_info[0]);i++) {
    names+=({ dir_info[0][i] });
  }
  names=sort_array(names);
  actor->send_message(sprintf("%*#a", actor->query_cols(), names)+"\n");
}

void ls_path(object actor, string verb, object *x, string path) {
  mixed **dir_info;
  int i;
  string *names;

  dir_info=get_dir(path);
  if (sizeof(dir_info[0])==1 && dir_info[1][0]==-2) {
    dir_info=get_dir(path+"/*");
  }
  names=({ });
  for (i=0; i<sizeof(dir_info[0]);i++) {
    names+=({ dir_info[0][i] });
  }
  names=sort_array(names);
  actor->send_message(sprintf("%*#a", actor->query_cols(), names)+"\n");
}

void man(object actor, string verb, object *x, string subject) {
  string filename;
  string text;
  mixed dir_info;

  filename=find_file(subject, "/doc/man");
  if (filename) {
    dir_info=get_dir(filename);
    if (dir_info[1][0]<0) {
      /* It's a directory */
      dir_info=get_dir(filename+"/*");
      text=sprintf("%*#a\n", actor->query_cols(), dir_info[0]);
    } else {
      /* It's a file */
      text=read_file(filename);
    }
    if (text) {
      actor->more_string(text);
    }
  } else {
    actor->send_message("No man page available for "+subject+".\n");
  }
}

void edit(object actor, string verb, object *x, string path) {
  actor->edit(actor->real_dir(path));
}

void more_file(object actor, string verb, object *x, string path) {
  string text;
  mixed *dir_info;

  dir_info=get_dir(path);
  if (sizeof(dir_info[0])==0) {
    actor->send_message("Can't find the file.\n");
  } else if (dir_info[1][0]<0) {
    actor->send_message("That's a directory.\n");
  } else {
    text=read_file(path);
    actor->more_string(text);
  }
}

void more_object(object actor, string verb, object *objs, string x) {
  string text;
  string path, dummy;

  path=object_name(objs[0]);
  sscanf(path, "%s#%s", path, dummy);
  text=path+"\n\n"+read_file(path+".c");
  actor->more_string(text);
}

void mkdir(object actor, string verb, object *x, string path) {
  make_dir(path);
}

void rmdir(object actor, string verb, object *x, string path) {
  remove_dir(path);
}
