#include "kernel/physical.h"

inherit "/kernel/lib/command";

void init(object person) {
  ::init(person);
  person->add_grammar("upgrade", "%file", "upgrade", ({ 0 }) );
  person->add_grammar("upgrade", "%object", "upgrade_obj", ({ 0 }) );
  person->add_grammar("upgrade", "", "auto_upgrade", ({ }) );
  person->add_grammar("clone", "%file", "clone", ({ 0 }) );
  person->add_grammar("dest", "%multi", "dest", ({ 0 }) );
  person->add_grammar("set_property", "%object %word %word", "set_prop", ({ 0, 1, 2 }) );
  person->add_grammar("query_property", "%object %word", "query_prop", ({ 0, 1 }) );
}

void report_upgrade(object actor, string *paths) {
  int i;

  actor->send_message("Upgraded: \n");
  if (sizeof(paths)) {
    for (i=0;i<sizeof(paths);i++) {
      actor->send_message("  "+paths[i]+"\n");
    }
  } else {
    actor->send_message("  Nothing.\n");
  }
}

void upgrade(object actor, string verb, object *x, string path) {
  string *done;

  done="/kernel/sys/driver"->upgrade(path);
  report_upgrade(actor, done);
}

void upgrade_obj(object actor, string verb, object *obj, string x) {
  string path;
  string *done;
  int i;

  done=({ });
  for (i=0;i<sizeof(obj);i++) {
    path=explode(object_name(obj[i]), "#")[0];
    done+="/kernel/sys/driver"->upgrade(path);
  }
  report_upgrade(actor, done);
}

void auto_upgrade(object actor, string verb) {
  string *done;

  done="/kernel/sys/driver"->upgrade(nil);
  report_upgrade(actor, done);
}

void clone(object actor, string verb, object *x, string path) {
  object obj;

  if (sizeof(get_dir(path+".c")[1]) && get_dir(path+".c")[1][0]>0) {
    obj=find_object(path);
    if (obj) {
      obj=clone_object(obj);
      if (obj) {
        obj->move(actor, actor);
        obj->set_property(PROP_RELATION, REL_HELD);
      }
    }
    if (obj) {
      actor->send_message("Object cloned\n");
    } else {
      actor->send_message("Sorry - couldn't clone it.\n");
    }
  } else {
    actor->send_message("Can't find object\n");
  }
}

void dest(object actor, string verb, object *objs, string x) {
  int i;

  for (i=0; i<sizeof(objs); i++) {
    if (objs[i]!=actor) { /* Don't let them suicide... */
      destruct_object(objs[i]);
    }
  }
}

void set_prop(object actor, string verb, object *objs, object *x, object *y, string z, string property, string value) {
  string strval;
  int intval;

  if (sscanf(value, "\"%s\"", strval)==1) {
    objs[0]->set_property(property, strval);
  } else if (sscanf(value, "%d", intval)==1) {
    objs[0]->set_property(property, intval);
  } else if (value=="nil") {
    objs[0]->set_property(property, nil);
  }
}

void query_prop(object actor, string verb, object *objs, object *x, string y, string property) {
  actor->send_message(mixed_to_string(objs[0]->query_property(property))+"\n");
}
