#include <kernel/physical.h>
#define OBJECTS_DATA "/data/dispenser.dat"

inherit physical "/kernel/lib/physical";

mapping global_objects;
mapping private_objects;

void load_objects(string file, object person, int is_private) {
  string *data;
  string key, info;
  int i;
  mixed *file_info;

  file_info=get_dir(file);
  if (sizeof(file_info[0])) {
    depends_on(file);
    data=explode(read_file(file), "\n");
    for (i=0;i<sizeof(data);i++) {
      if (data[i]!="" && data[i][0]!='#') {
        sscanf(data[i], "%s|%s", key, info);
        if (is_private) {
          if (!private_objects[person]) {
            private_objects[person]=([ ]);
          }
          private_objects[person][key]=info;
        } else {
          global_objects[key]=info;
        }
      }
    }
  }
}

void create() {
  physical::create();
  set_ids(({ "dispenser", "dispensing machine", "machine" }));
  set_name("dispensing machine");
  set_description("A machine for dispensing random things");
  global_objects=([ ]);
  private_objects=([ ]);
  load_objects(OBJECTS_DATA, nil, 0);
}

void upgraded() {
  global_objects=([ ]);
  private_objects=([ ]);
  load_objects(OBJECTS_DATA, nil, 0);
}

void init(object person) {
  physical::init(person);
  if (!person->query_property("banish_from_dispenser")) {
    if (person->query_home_dir()) {
      load_objects(person->query_home_dir()+"/data/dispenser.dat", person, 1);
    }
    person->add_grammar("grab", "%text", "do_grab", ({ 0 }) );
    person->add_grammar("grab", "%text from %thisobject", "do_grab", ({ 0 }) );
  }
}

void uninit(object person) {
  private_objects[person]=nil;
}

object make_object(string data, object actor) {
  string *info;
  object obj;
  mixed *file_info;

  /* format of the data line:
     0         1   2    3           4...
     file name|ids|name|description|other data */
  info=explode(data, "|");
  file_info=get_dir(info[0]+".c");
  if (sizeof(file_info[0])) {
    obj=find_object(info[0]);
    if (obj) {
      obj=clone_object(obj);
      obj->set_ids(explode(info[1], ":"));
      obj->set_name(info[2]);
      obj->set_description(info[3]);
      if (info[0]=="/kernel/obj/food_drink") {
        obj->set_property("food_type", (int)info[4]);
        obj->set_size((int)info[5]);
      }
      obj->move(actor, actor);
      obj->set_property(PROP_RELATION, REL_HELD);
    } else {
      actor->send_message("Sorry - that object is out of stock.\n");
    }
  } else {
    actor->send_message("Sorry - that object is out of stock.\n");
  }
}

int do_grab(object actor, string verb, object *x, string text) {
  object obj;
  string *bits;
  string name, description;

  if (global_objects[text]) {
    make_object(global_objects[text], actor);
  } else if (private_objects[actor] && private_objects[actor][text]) {
    make_object(private_objects[actor][text], actor);
  } else {
    bits=explode(text, ":");
    obj=find_object("/kernel/obj/generic_object");
    if (obj) {
      obj=clone_object(obj);
      obj->set_ids(bits[0]);
      obj->set_name(bits[0]);
      if (sizeof(bits)>1) {
        obj->set_description(bits[1]);
      } else {
        obj->set_description(capitalize(obj->query_art_name())+".");
      }
      obj->move(actor, actor);
      obj->set_property(PROP_RELATION, REL_HELD);
    }
  }
}
