#include "/kernel/user.h"
#include "/kernel/physical.h"

inherit living "/kernel/lib/living";
inherit arrays "/kernel/lib/arrays";

object flirtee;
int flirt_callout;
object last_victim;

void create() {
  living::create();
  set_name("idle policeman");
  set_article("the");
  set_description("A brave young policeman, dedicated to tidying up the world "+
                  "by eliminating wanton idlers...");
  set_ids( ({"policeman", "idle police", "police", "idle policeman"}) );
  call_out("check_idle", 60);
  flirt_callout=call_out("do_flirt", 5 * 60 );
  "/kernel/command/soul"->init(this_object());
}

void init(object person) {
  living::init(person);
  person->add_grammar("seduce", "%thisobject", "do_seduce", ({ 0 }) );
  person->add_grammar("arrest", "%person", "do_arrest", ({ 0 }) );
}

void setup_flirt_call() {
  remove_call_out(flirt_callout);
  /* Between half an hour and two hours... */
  flirt_callout=call_out("do_flirt", (30+random(90))*60);
}

void do_flirt_command(object person) {
  execute_command(({ "melt", "flirt", "snuggle", "kiss" })[random(4)]+" "+
                  person->query_name());
  if (person==flirtee) {
    setup_flirt_call();
  }
}

void do_seduce(object actor, string verb, object *x, string y) {
  if (!flirtee) {
    send_event(actor, "action", "You", "seduce the policeman.");
    send_event(actor->near_objects()-({actor}), "action",
               actor->query_name(), "seduces the policeman.");
    do_flirt_command(actor);
    flirtee=actor;
  } else {
    actor->send_message("It doesn't work - the idle policeman is on duty...\n");
    send_event(actor->near_objects()-({actor}), "action",
               actor->query_name(), "fails to seduce the idle policeman.");
  }
}

void do_grab(object target) {
  if (query_environment()!=target->query_environment()) {
    send_event(near_objects(), "action", query_art_name(), 
               "rushes out of the room.");
    move(target->query_environment(), this_object());
  }
  send_event(near_objects()-({target}), "action", query_art_name(),
             "grabs "+target->query_name()+" by the feet and drags "+
             target->query_pronoun(PRON_SUBJ)+" off to the Idle Tank to sleep "+
             "it off.");
  send_event(target, "action", query_art_name(),
             "grabs you by the feet and drags you off to the Idle Tank to "+
             "sleep it off.");
  last_victim=target;
  call_out("tidyup1",2);
  move("/rooms/idle_tank", this_object());
  target->move("/rooms/idle_tank", this_object());
}

void do_arrest(object actor, string verb, object *victim, string y) {
  do_grab(victim[0]);
} 

void event_soul(object sender, object source, string verb, string adverb, object *targets, string text) {
  if ((source!=this_object()) && (member_array(this_object(), targets)!=-1)) {
    call_out("do_response", 2, source, verb, adverb, targets, text);
  }
}

void do_response(object source, string verb, string adverb, object *targets, string text) {
  switch (verb) {
    case "kick": case "smack": case "knee": case "spit": case "slap": 
    case "punch": case "jump":
      if (source==flirtee) {
        execute_command("accuse "+source->query_name()+" of breaking his heart");
        flirtee=nil;
      }
      do_grab(source);
      break;
    case "scold": case "waggle" :
      execute_command("pout "+source->query_name());
      break;
    case "pat" :
      if (source==flirtee) {
        do_flirt_command(flirtee);
      } else {
        execute_command("bite "+source->query_name());
      }
      break;
    case "ignore" :
      if (source==flirtee) {
        execute_command("accuse "+source->query_name()+" of breaking his heart");
        flirtee=nil;
      }
      execute_command("sulk");
      break;
    case "french" :
      execute_command("gasp");
      if (source==flirtee) {
        execute_command("french "+source->query_name());
        setup_flirt_call();
      } else {
        execute_command("slap "+source->query_name());
      }
      break;
    case "hug": case "stare": case "cuddle": case "gaze": case "adore": 
    case "flirt": case "kiss": case "fondle": case "nuzzle": 
    case "caress": case "stroke" :
      if (flirtee==source) {
        do_flirt_command(flirtee);
      } else {
        execute_command("hug "+source->query_name()+" tightly");
        execute_command("whisper I'm currently on the job.  If you would like to "+
                   "come back tonight... I'll try to fit you in to "+
                   source->query_name());
      }
      break;
    case "admire" :
      execute_command("smile knowingly "+source->query_name());
      break;
    case "glare" :
      execute_command("lick "+source->query_name());
      break;
    default :
      if (flirtee==source) {
        do_flirt_command(flirtee);
      } else {
        execute_command("ignore "+source->query_name());
        send_event(source->near_objects(), "action", query_art_name(), "is on duty.");
      }
  }    
}

void check_idle() {
  object *people;
  object room;
  mixed info;
  int i;

  call_out("check_idle", 60);
  people=users();
  for (i=0;i<sizeof(people);i++) {
    info=people[i]->query_user_info();
    room=people[i]->query_environment();
    if (info[USER_INFO_IDLE]>=123*60 && 
        !(room && room->query_name() == "Idle Tank")) {
      do_grab(people[i]);
      return;
    }
  }
}

void tidyup1() {
  object *people;
  mixed *info;
  object next_victim;
  int i, maxidle;

  send_event(last_victim, "action", "The cell door", "slams shut.");
  send_event(all_objects()-({last_victim}), "action", "A cell door", "slams, off "+
             "in the distance...");
  people=users();
  next_victim=nil;
  maxidle=0;
  for (i=0;i<sizeof(people);i++) {
    info=people[i]->query_user_info();
    if (info[USER_INFO_IDLE]>maxidle && 
        people[i]->query_environment()->query_name()!="Idle Tank") {
      next_victim=people[i];
      maxidle=info[USER_INFO_IDLE];
    }
  }
  if (next_victim) {
    call_out("tidyup2", 5, next_victim);
  }
}

void tidyup2(object next_victim) {
  move(next_victim->query_environment(), this_object());
  send_event(near_objects()-({next_victim}), "action", query_art_name(),
             "strolls back into the room and gives "+next_victim->query_name()+
             " a suspicious glare.");
  send_event(next_victim, "action", query_art_name(),
             "strolls back into the room and gives you a suspicious glare.");
}

void do_flirt() {
  object *people;
  int i;
  mixed info;
  object next_flirtee;
  int min_time;

  min_time=30*60;
  next_flirtee=nil;
  setup_flirt_call();
  people=users();
  for (i=0;i<sizeof(people);i++) {
    info=people[i]->query_user_info();
    if (info[USER_INFO_IDLE]<min_time) {
      min_time=info[USER_INFO_IDLE];
      next_flirtee=people[i];
    }
  }
  if (next_flirtee) {
    flirtee=next_flirtee;
    do_flirt_command(flirtee);
  }
}

/* 
soul commands needed by the policeman: 

accuse admire   adore bite  caress cuddle *flirt
fondle french   gasp  gaze  glare  hug    ignore
jump   kick     *kiss knee  lick   *melt  nuzzle
pat    pout     punch scold slap   slap   smack
smile  *snuggle spit  stare stroke sulk   waggle
*/
