#define CLOTH_IDS 0
#define CLOTH_NAME 1
#define CLOTH_DESCRIPTION 2

#include "/kernel/physical.h"
#include "limits.h"

inherit physical "/kernel/lib/physical";
inherit sprintf "/kernel/lib/sprintf";
inherit arrays "/kernel/lib/arrays";

mapping private_clothes; /* name->clothing data */
mixed *global_clothes; /* string *ids, string name, string description */

mixed *get_data(mixed *data, string text) {
  int i, j;

  if (data) {
    for (i=0; i<sizeof(data); i++) {
      for (j=0; j<sizeof(data[i][CLOTH_IDS]); j++) {
        if (text==data[i][CLOTH_IDS][j]) {
          return data[i];
        }
      }
    }
  }
  return nil;
}

mixed get_objs_from_clothing(object actor, string text, mixed *grammar_line) {
  if (get_data(private_clothes[actor], text)) {
    return ({ 1, ({ }) });
  }  else if (get_data(global_clothes, text)) {
    return ({ 1, ({ }) });
  } else {
    return ({ -1000, ({ }) });
  }
}

string query_description() {
  object actor;
  mixed *clothes;
  int i;
  string result;
  string *contents;
  int num_columns;
  int best_size, best_cols;
  string name;

  actor=this_user();
  clothes=global_clothes;
  if (actor && private_clothes[actor]) {
    clothes+=private_clothes[actor];
  }
  contents=({ });
  for (i=0; i<sizeof(clothes); i++) {
    name=clothes[i][CLOTH_NAME];
    if (sizeof(explode(name, "#"))>1) {
      name=explode(name, "#")[1]+" "+explode(name, "#")[0];
    } else {
      switch (lower_case(name)[0]) {
	case 'a': case 'e': case 'i': case 'o': case 'u':
	  name="an "+name;
	  break;
	default:
	  name="a "+name;
	  break;
      }
    }
    contents+=({ clothes[i][CLOTH_IDS][0]+": "+capitalize(name)+"\n" });
  }
  contents=sort_array(contents);
  best_size=INT_MAX;
  best_cols=2;
  for(num_columns=1;num_columns<5;num_columns++) {
    result=sprintf("%*=i-2s", actor->query_cols()/num_columns-2, implode(contents, "\n"));
    result=sprintf("%*.*#a", actor->query_cols(), num_columns, explode(result, "\n"));
    if (sizeof(explode(result, "\n"))<best_size) {
      best_size=sizeof(explode(result, "\n"));
      best_cols=num_columns;
    }
  }
  result=sprintf("%*=i-2s", actor->query_cols()/best_cols-2, implode(contents, "\n"));
  result=sprintf("%*.*#a", actor->query_cols(), best_cols, explode(result, "\n"));
  result=physical::query_description()+"  Inside, you can see:\n"+result;
  return result;
}

void add_data(object person, string filename, int is_private) {
  string *infile;
  string *ids;
  string name, description;
  int i;

  if (sizeof(get_dir(filename)[0])) {
    infile=explode(read_file(filename), "\n");
    i=0;
    while(i<sizeof(infile)) {
      ids=explode(infile[i++], "#");
      name=infile[i++];
      description="";
      while (i<sizeof(infile) && infile[i]!="") {
        if (description!="") {
          description+=" ";
        }
        description+=infile[i++];
      }
      if (is_private) {
        if (!private_clothes[person]) {
	  private_clothes[person]=({ });
        }
        private_clothes[person]+=({ ({ids, name, description}) });
      } else {
        global_clothes+=({ ({ids, name, description}) });
      }
      i++;
    }
  }
}

void do_borrow(object actor, string verb, object *x, string id) {
  object obj;
  string name;
  mixed *data;

  data=get_data(private_clothes[actor], id);
  if (!data) {
    data=get_data(global_clothes, id);
  }
  if (data) {
    obj=find_object("/kernel/obj/clothing");
    if (obj) {
      obj=clone_object(obj);
      name=data[CLOTH_NAME];
      if (sizeof(explode(name, "#"))>1) {
        obj->set_article(explode(name, "#")[1]);
        name=explode(name, "#")[0];
      }
      obj->set_desc(name, data[CLOTH_DESCRIPTION], data[CLOTH_IDS]);
      obj->move(actor, actor);
      obj->set_property(PROP_RELATION, REL_HELD);
      send_event(actor, "action", "You", "borrow "+obj->query_art_name()+
        " from the wardrobe.");
      send_event(actor->near_objects()-({actor}), 
        "action", actor->query_name(), "borrows "+obj->query_art_name()+
        " from the wardrobe.");
    }
  }
}

void do_make(object actor, string verb) {
  object maker;

  maker=find_object("/obj/clothing_maker");
  maker=clone_object(maker);
  maker->new_clothing(actor, this_object());
}

void create() {
  physical::create();
  set_ids("wardrobe");
  set_name("wardrobe");
  set_description("A wardrobe full of clothes.");
  private_clothes=([ ]);
  global_clothes=({ });
  add_data(nil, "/data/clothes.dat", 0);
  depends_on("/data/clothes.dat");
}

void init(object person) {
  physical::init(person);
  if (person->query_home_dir()) {
    add_data(person, person->query_home_dir()+"/data/clothes.dat", 1);
  }
  person->add_grammar("borrow", "%clothing", "do_borrow", ({ 0 }) );
  person->add_grammar("make_clothes", "", "do_make", ({ }) );
}

void uninit(object person) {
  private_clothes[person]=nil;
  ::uninit(person);
}
